package org.seasar.javelin.jmx.viewer.editpart;

import java.beans.PropertyChangeEvent;
import java.util.List;

import org.eclipse.draw2d.ChopboxAnchor;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.Layer;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.ToolbarLayout;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.eclipse.swt.graphics.Color;
import org.seasar.javelin.jmx.viewer.editors.EditPartWithListener;
import org.seasar.javelin.jmx.viewer.model.ComponentModel;
import org.seasar.javelin.jmx.viewer.model.InvocationModel;


public class ComponentEditPart extends EditPartWithListener implements
		NodeEditPart
{
	/** NX̔wiFB */
	private static final Color YELLOW = new Color(null,255, 255, 206);
	
	/** xFB */
	private static final Color ORANGE = new Color(null,224, 160,   0);
	
	protected IFigure createFigure()
	{
		ComponentModel model = (ComponentModel) getModel();

	    Layer layer = new Layer();
	    layer.setBorder(new LineBorder());
	    layer.setOpaque(true);
	    layer.setBackgroundColor(YELLOW);
	    layer.setLayoutManager(new ToolbarLayout());

	    // R|[lgpx̐ݒB
		Label label = new Label();

		// wiFF
		label.setBackgroundColor(YELLOW);

		// wiFs
		label.setOpaque(true);
		
		// \̌
		String componentName = "";
		if (model.getClassName().indexOf("/") > -1)
		{
//			componentName = 
//				model.getClassName().substring(
//					model.getClassName().lastIndexOf("/") + 1);
			componentName = model.getClassName();
		}
		else
		{
			componentName = 
				model.getClassName().substring(
					model.getClassName().lastIndexOf(".") + 1);
		}
		
		label.setText(componentName);
		layer.add(label);
		
		CompartmentFigure figure = new CompartmentFigure();
		figure.setBackgroundColor(YELLOW);
		
		for (InvocationModel invocation : model.getInvocationList())
		{
			Label invocationLabel = new Label();
			invocationLabel.setOpaque(true);
			invocationLabel.setBackgroundColor(YELLOW);
			invocationLabel.setTextAlignment(PositionConstants.ALWAYS_LEFT);
			invocationLabel.setText(
					invocation.getMethodName() 
					+ "(" 
					+ invocation.getMaximum() 
					+ ":"
					+ invocation.getAverage()
					+ ")");
			if (invocation.getMaximum() > invocation.getAlarmThreshold())
			{
				invocationLabel.setForegroundColor(ColorConstants.red);
			}
			else if (invocation.getMaximum() > invocation.getWarningThreshold())
			{
				invocationLabel.setForegroundColor(ORANGE);
			}
			
			figure.add(invocationLabel);
		}
		
		layer.add(figure);
		
		return layer;
	}

	public ConnectionAnchor getSourceConnectionAnchor(Request request)
	{
		return new ChopboxAnchor(getFigure());
	}

	public ConnectionAnchor getTargetConnectionAnchor(Request request)
	{
		return new ChopboxAnchor(getFigure());
	}

	public ConnectionAnchor getSourceConnectionAnchor(
			ConnectionEditPart connection)
	{
		return new ChopboxAnchor(getFigure());
	}

	public ConnectionAnchor getTargetConnectionAnchor(
			ConnectionEditPart connection)
	{
		return new ChopboxAnchor(getFigure());
	}

	protected void refreshVisuals()
	{
		// ̎擾
		Rectangle constraint = ((ComponentModel) getModel()).getConstraint();

		// RectangleIuWFNg𐧖Ƃăr[ɐݒ肷
		// setLayoutConstraint\bh͐eEditPartĂяo
		((GraphicalEditPart) getParent()).setLayoutConstraint(this,
				getFigure(), constraint);
	}

	protected void createEditPolicies()
	{
	}

	public void propertyChange(PropertyChangeEvent evt)
	{
		// ύX̌^f̈ʒu̕ύX̂ǂ
		if (evt.getPropertyName().equals(ComponentModel.P_CONSTRAINT))
			refreshVisuals(); // r[XV
		else if (evt.getPropertyName().equals(
				ComponentModel.P_SOURCE_CONNECTION))
			refreshSourceConnections(); // RlNVE\[X̍XV
		else if (evt.getPropertyName().equals(
				ComponentModel.P_TARGET_CONNECTION))
			refreshTargetConnections(); // RlNVE^[Qbg̍XV
	}

	protected List getModelSourceConnections()
	{
		// EditPartڑƂRlNVEf̃XgԂ
		return ((ComponentModel) getModel()).getModelSourceConnections();
	}

	protected List getModelTargetConnections()
	{
		// EditPartڑƂRlNVEf̃XgԂ
		return ((ComponentModel) getModel()).getModelTargetConnections();
	}
}
